package de.hdm_stuttgart.battlearena.Model.Entity;

import de.hdm_stuttgart.battlearena.Controller.Enum.PlayerMode;
import de.hdm_stuttgart.battlearena.Controller.GameSceneController;
import de.hdm_stuttgart.battlearena.Model.DataStorage.Classes.RuntimeInfo;
import de.hdm_stuttgart.battlearena.Model.Entity.GameplayObjects.ObjectStatus;
import de.hdm_stuttgart.battlearena.Model.Entity.GameplayObjects.ObjectType;
import de.hdm_stuttgart.battlearena.Model.Inputs.InputHandler;
import de.hdm_stuttgart.battlearena.Model.Map.TileManager;
import de.hdm_stuttgart.battlearena.Model.Sound.SFX;
import de.hdm_stuttgart.battlearena.Model.Sound.SoundEffects;

import javafx.geometry.BoundingBox;
import javafx.scene.canvas.GraphicsContext;
import javafx.scene.image.Image;

import org.apache.logging.log4j.Logger;
import org.apache.logging.log4j.LogManager;

import java.util.List;
import java.util.Objects;


class Player implements IEntity {

    private static final Logger log = LogManager.getLogger(Player.class);

    private final PlayerMode PLAYER_MODE;

    SoundEffects soundEffects = new SoundEffects();

    InputHandler inputHandler;

    RuntimeInfo runtimeInfo = RuntimeInfo.getInstance();

    CollisionHandler collisionHandler = new CollisionHandler();

    GraphicsContext gameScene;

    GameSceneController gameSceneController;

    private BoundingBox boxCollider;

    private Image directionDownOne,
            directionDownTwo,
            directionUpOne,
            directionUpTwo,
            directionLeftOne,
            directionLeftTwo,
            directionRightOne,
            directionRightTwo;

    private Image swordDown,
            swordUp,
            swordLeft,
            swordRight;

    private final EntityClass entityClass;

    private int spriteCounter = 0;
    private int spriteNumber = 1;

    private final int playerWidth = 19;
    private final int playerHeight = 20;

    private int scaledTileSize;

    private int mapPosX;
    private int mapPosY;

    private int lastMapPosX;
    private int lastMapPosY;

    private int playerSpeed;
    private EntityDirection playerDirection;
    private int health;
    private int maxPlayerHealth;
    private int damage;
    private int iFrameCounter = 0;
    private int attackRate = 0;
    private int bombPlacementRate = 0;
    private boolean isBigBombItemFrame = false;
    private boolean isBlastUpItemFrame = false;
    private int blastradius = 1;

    public Player(GraphicsContext gameScene, InputHandler inputHandler, EntityClass entityClass,
                  GameSceneController gameSceneController, PlayerMode PLAYER_MODE) {
        this.gameScene = gameScene;
        this.inputHandler = inputHandler;
        this.entityClass = entityClass;
        this.gameSceneController = gameSceneController;
        this.PLAYER_MODE = PLAYER_MODE;

        initializeEntity();
        loadEntitySprites();
    }

    @Override
    public void initializeEntity() {
        scaledTileSize = gameSceneController.getScaledTileSize();
        if (PLAYER_MODE == PlayerMode.PLAYER_ONE) {
            mapPosX = 60;
            mapPosY = 60;
        } else if (PLAYER_MODE == PlayerMode.PLAYER_TWO) {
            mapPosX = 760;
            mapPosY = 735;
        }

        initializePlayerStats();

        boxCollider = new BoundingBox(mapPosX + 15, mapPosY + 10, playerWidth, playerHeight);
        playerDirection = EntityDirection.DOWN;
    }

    //Values result from balancing
    private void initializePlayerStats() {
        if (entityClass == EntityClass.HUMAN) {
            maxPlayerHealth = 50;
            health = 50;
            damage = 5;
            playerSpeed = 4;
        } else if (entityClass == EntityClass.HIGH_BORN) {
            maxPlayerHealth = 100;
            health = 75;
            damage = 2;
            playerSpeed = 3;
        } else if (entityClass == EntityClass.LOW_BORN) {
            maxPlayerHealth = 10;
            health = 10;
            damage = 10;
            playerSpeed = 7;
        } else {
            maxPlayerHealth = 40;
            health = 15;
            damage = 7;
            playerSpeed = 5;
        }
    }

    @Override
    public void loadEntitySprites() {
        try {
            if (entityClass == EntityClass.HUMAN) {
                loadHumanSprites();
                loadWeaponSprites();
            } else if (entityClass == EntityClass.HIGH_BORN) {
                loadHighBornSprites();
                loadWeaponSprites();
            } else if (entityClass == EntityClass.LOW_BORN) {
                loadLowBornSprites();
                loadWeaponSprites();
            } else {
                loadSentinelsSprites();
                loadWeaponSprites();
            }
        } catch (Exception e) {
            log.error(e);
        }
    }

    private void loadHumanSprites() {
        directionDownOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/human/HumanDown00.png")));
        directionDownTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/human/HumanDown01.png")));
        directionUpOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/human/HumanUp00.png")));
        directionUpTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/human/HumanUp01.png")));
        directionLeftOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/human/HumanLeft00.png")));
        directionLeftTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/human/HumanLeft01.png")));
        directionRightOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/human/HumanRight00.png")));
        directionRightTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/human/HumanRight01.png")));
    }

    private void loadHighBornSprites() {
        directionDownOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/high_born/HighBornDown00.png")));
        directionDownTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/high_born/HighBornDown01.png")));
        directionUpOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/upOne.png")));
        directionUpTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/upTwo.png")));
        directionLeftOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/leftOne.png")));
        directionLeftTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/leftTwo.png")));
        directionRightOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/rightOne.png")));
        directionRightTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/rightTwo.png")));
    }

    private void loadLowBornSprites() {
        directionDownOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/HumanDown01.png")));
        directionDownTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/HumanDown02.png")));
        directionUpOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/upOne.png")));
        directionUpTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/upTwo.png")));
        directionLeftOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/leftOne.png")));
        directionLeftTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/leftTwo.png")));
        directionRightOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/rightOne.png")));
        directionRightTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/rightTwo.png")));
    }

    private void loadSentinelsSprites() {
        directionDownOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/sentinels/SentinelDown00.png")));
        directionDownTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/sentinels/SentinelDown01.png")));
        directionUpOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/upOne.png")));
        directionUpTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/upTwo.png")));
        directionLeftOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/leftOne.png")));
        directionLeftTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/leftTwo.png")));
        directionRightOne = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/rightOne.png")));
        directionRightTwo = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/player/rightTwo.png")));
    }

    private void loadWeaponSprites() {
        swordUp = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/weapons/SwordUp.png")));
        swordDown = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/weapons/SwordDown.png")));
        swordLeft = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/weapons/SwordLeft.png")));
        swordRight = new Image(Objects.requireNonNull(getClass().getResourceAsStream("/textures/weapons/SwordRight.png")));
    }

    @Override
    public void updateEntity(GameSceneController gameScene) {
        if (PLAYER_MODE == PlayerMode.PLAYER_ONE) {
            updateInvincibility();
            updateAttackRate();
            updateBombPlacementRate();
            if (inputHandler.isMoveUp() || inputHandler.isMoveDown() || inputHandler.isMoveLeft() || inputHandler.isMoveRight()) {
                if (inputHandler.isMoveUp()) {
                    playerDirection = EntityDirection.UP;
                } else if (inputHandler.isMoveDown()) {
                    playerDirection = EntityDirection.DOWN;
                } else if (inputHandler.isMoveLeft()) {
                    playerDirection = EntityDirection.LEFT;
                } else if (inputHandler.isMoveRight()) {
                    playerDirection = EntityDirection.RIGHT;
                }

                boolean isWalkableTile = collisionHandler.handleMapCollision(this, scaledTileSize, gameSceneController);

                performEntityMovement(isWalkableTile);

                boolean isEntityCollision = boxCollider.intersects(gameScene.getEnemy().getBoxCollider());
                //boolean isEntityCollision = collisionHandler.handleBoxCollision(boxCollider, gameScene.getEnemy().getBoxCollider());

                performEntityCollision(isEntityCollision);

                updateEntityWalkAnimation();
            }
        } else if (PLAYER_MODE == PlayerMode.PLAYER_TWO) {
            updateInvincibility();
            updateAttackRate();
            updateBombPlacementRate();
            if (inputHandler.isSdMoveUp() || inputHandler.isSdMoveDown() || inputHandler.isSdMoveLeft() || inputHandler.isSdMoveRight()) {
                if (inputHandler.isSdMoveUp()) {
                    playerDirection = EntityDirection.UP;
                } else if (inputHandler.isSdMoveDown()) {
                    playerDirection = EntityDirection.DOWN;
                } else if (inputHandler.isSdMoveLeft()) {
                    playerDirection = EntityDirection.LEFT;
                } else if (inputHandler.isSdMoveRight()) {
                    playerDirection = EntityDirection.RIGHT;
                }

                boolean isWalkableTile = collisionHandler.handleMapCollision(this, scaledTileSize, gameSceneController);

                performEntityMovement(isWalkableTile);

                boolean isEntityCollision = boxCollider.intersects(gameScene.getPlayer().getBoxCollider());
                //boolean isEntityCollision = collisionHandler.handleBoxCollision(boxCollider, gameScene.getEnemy().getBoxCollider());

                performEntityCollision(isEntityCollision);

                updateEntityWalkAnimation();
            }
        }
    }

    private void updateInvincibility() {
        if (iFrameCounter > 0) {
            iFrameCounter--;
        }
    }

    private void updateAttackRate() {
        if (attackRate > 0) {
            attackRate--;
        }
    }

    private void updateBombPlacementRate() {
        if (bombPlacementRate > 0) {
            bombPlacementRate--;
        }
    }

    private void performEntityMovement(boolean isWalkableTile) {
        lastMapPosX = mapPosX;
        lastMapPosY = mapPosY;

        if (isWalkableTile) {
            switch (playerDirection) {
                case UP:
                    mapPosY -= playerSpeed;
                    break;
                case DOWN:
                    mapPosY += playerSpeed;
                    break;
                case LEFT:
                    mapPosX -= playerSpeed;
                    break;
                case RIGHT:
                    mapPosX += playerSpeed;
            }
        }
    }

    private void performEntityCollision(boolean isEntityCollision) {
        if (isEntityCollision) {
            soundEffects.playSoundEffectOnce(SFX.COLLISION);
            if (playerDirection == EntityDirection.UP) {
                mapPosY = lastMapPosY + 30;
            } else if (playerDirection == EntityDirection.DOWN) {
                mapPosY = lastMapPosY - 30;
            } else if (playerDirection == EntityDirection.LEFT) {
                mapPosX = lastMapPosX + 30;
            } else {
                mapPosX = lastMapPosX - 30;
            }
        }

        boxCollider = new BoundingBox(mapPosX + 15, mapPosY + 20, playerWidth, playerHeight);
    }

    @Override
    public void checkGameplayObjectInteraction(IEntity entity, GraphicsContext graphicsContext) {

        List<IEntity> gameplayObjects = runtimeInfo.getGameplayObjects();
        for (IEntity gameplayObject : gameplayObjects) {
            if (gameplayObject.getOBJECT_TYPE() == ObjectType.HEART) {
                if (gameplayObject.getBoxCollider().intersects(boxCollider)) {
                    log.debug("Collision with heart");
                    healPlayer(5);
                    gameplayObject.setObjectStatus(ObjectStatus.USED);
                }
            } else if (gameplayObject.getOBJECT_TYPE() == ObjectType.BOMB) {
                if (gameplayObject.getBoxCollider().intersects(boxCollider)) {
                    log.debug("Collision with bomb");
                    gotHit(10);
                }
            } else if (gameplayObject.getOBJECT_TYPE() == ObjectType.EXPLOSION) {
                if (gameplayObject.getBoxCollider().intersects(boxCollider)) {
                    log.debug("Collision with explosion");
                    gotHit(10);
                }
            } else if (gameplayObject.getOBJECT_TYPE() == ObjectType.BIG_BOMB) {
                if (gameplayObject.getBoxCollider().intersects(boxCollider)) {
                    log.debug("Collision with bigBomb");
                    gotHit(10);
                }
            } else if (gameplayObject.getOBJECT_TYPE() == ObjectType.BIG_BOMB_ITEMFRAME) {
                if (gameplayObject.getBoxCollider().intersects(boxCollider)) {
                    isBigBombItemFrame = true;
                    log.info("Collision with bigBombItemFrame");
                    gameplayObject.setObjectStatus(ObjectStatus.USED);
                }
            } else if (gameplayObject.getOBJECT_TYPE() == ObjectType.BLASTUP_ITEMFRAME) {
                if (gameplayObject.getBoxCollider().intersects(boxCollider)) {
                    isBlastUpItemFrame = true;
                    log.info("Collision with blastUpItemFrame");
                    gameplayObject.setObjectStatus(ObjectStatus.USED);
                }
            }
        }
    }

    public int updateblastradius(int blastradius) {
        if (isBlastUpItemFrame) {
            blastradius++;
            log.info("Upded blastradius:" + blastradius);
            isBlastUpItemFrame = false;
            return blastradius;
        } else {
            return blastradius;
        }
    }

    @Override
    public void placeBomb(IEntity entity, GraphicsContext graphicsContext) {
        blastradius = updateblastradius(blastradius);

        List<IEntity> gameplayObjects = runtimeInfo.getGameplayObjects();
        if (inputHandler.isBomb() && PLAYER_MODE == PlayerMode.PLAYER_ONE && bombPlacementRate == 0 ||
                inputHandler.isSdBomb() && PLAYER_MODE == PlayerMode.PLAYER_TWO && bombPlacementRate == 0) {

            if (isBigBombItemFrame) {
                log.debug("We have Itemframes in the List");
                bombPlacementRate = 100;
                int xTile = mapPosX / scaledTileSize;
                int yTile = mapPosY / scaledTileSize;

                if (TileManager.tileMap[yTile][xTile] < 10) {
                    int blastradiusbig = 18;
                    //List<IEntity> gameplayObjects = runtimeInfo.getGameplayObjects();
                    IEntity bigbomb = EntityFactory.createGameplayObject(ObjectType.BIG_BOMB, xTile * scaledTileSize, yTile * scaledTileSize, graphicsContext);
                    gameplayObjects.add(bigbomb);
                    runtimeInfo.setGameplayObjects(gameplayObjects);
                    log.info("Bigbomb placed");
                    checkExplosionSpawn(graphicsContext, blastradiusbig);
                    isBigBombItemFrame = false;
                }
            } else {
                log.debug("No Itemframes in the List");
                bombPlacementRate = 100;
                //int extrapixelpadding = 2;            //Could this fix the wrong Tile because of the Player??
                int xTile = mapPosX / scaledTileSize;
                int yTile = mapPosY / scaledTileSize;

                if (TileManager.tileMap[yTile][xTile] < 10) {
                    //List<IEntity> gameplayObjects = runtimeInfo.getGameplayObjects();
                    IEntity bomb = EntityFactory.createGameplayObject(ObjectType.BOMB, xTile * scaledTileSize, yTile * scaledTileSize, graphicsContext);
                    gameplayObjects.add(bomb);
                    runtimeInfo.setGameplayObjects(gameplayObjects);
                    log.info("bomb placed");
                    checkExplosionSpawn(graphicsContext, blastradius);
                }
            }
        }
    }

    public void checkExplosionSpawn(GraphicsContext graphicsContext, int blastradius) {
        int isDestructible;
        boolean stopDown = false, stopUp = false, stopRight = false, stopLeft = false;
        boolean stopUpRight = false, stopUpLeft = false, stopDownRight = false, stopDownLeft = false;

        int xTile = mapPosX / scaledTileSize;       //TODO: Fix sometimes bad bomb spawning
        int yTile = mapPosY / scaledTileSize;
        int yUp = yTile;
        int xRight = xTile;
        int yDown = yTile;
        int xLeft = xTile;
        //final int blastradius = 2;          //should be 1-2
        isDestructible = checkTilePlacing(yTile, xTile);
        if (isDestructible != 0) {
            for (int i = 0; i < blastradius; i++) {
                yDown += 1;
                yUp -= 1;
                xRight += 1;
                xLeft -= 1;
                if (yDown < 18 && yDown >= 0 && xTile < 18 && xTile >= 0) {
                    //changeCheckedTileDestroyable(graphicsContext);
                    isDestructible = checkTilePlacing(yDown, xTile);
                    if (isDestructible != 0 && !stopDown) {
                        spawnExplosion(yDown, xTile, graphicsContext);
                        if (isDestructible == 2) {
                            stopDown = true;
                        }
                    } else {
                        stopDown = true;
                    }
                }
                if (yUp < 18 && yUp >= 0 && xTile < 18 && xTile >= 0) {
                    //changeCheckedTileDestroyable(graphicsContext);
                    isDestructible = checkTilePlacing(yUp, xTile);
                    if (isDestructible != 0 && !stopUp) {
                        spawnExplosion(yUp, xTile, graphicsContext);
                        if (isDestructible == 2) {
                            stopUp = true;
                        }
                    } else {
                        stopUp = true;
                    }
                }
                if (yTile < 18 && yTile >= 0 && xRight < 18 && xRight >= 0) {
                    //changeCheckedTileDestroyable(graphicsContext);
                    isDestructible = checkTilePlacing(yTile, xRight);
                    if (isDestructible != 0 && !stopRight) {
                        spawnExplosion(yTile, xRight, graphicsContext);
                        if (isDestructible == 2) {
                            stopRight = true;
                        }
                    } else {
                        stopRight = true;
                    }
                }
                if (yTile < 18 && yTile >= 0 && xLeft < 18 && xLeft >= 0) {
                    //changeCheckedTileDestroyable(graphicsContext);
                    isDestructible = checkTilePlacing(yTile, xLeft);
                    if (isDestructible != 0 && !stopLeft) {
                        spawnExplosion(yTile, xLeft, graphicsContext);
                        if (isDestructible == 2) {
                            stopLeft = true;
                        }
                    } else {
                        stopLeft = true;
                    }
                }
            }
        }
    }

    public int checkTilePlacing(int y, int x) {     //Change it with a DestructionHandler like CollisionHandler //tileSet[xTile].getDestruction();
        if (TileManager.tileMap[y][x] >= 20) {
            return 2;       //Last spawn, in this direction
        } else if (TileManager.tileMap[y][x] <= 9) {
            return 1;       //Keep Going
        } else {
            return 0;       //dont continue
        }
    }

    public void changeCheckedTileDestructable(GraphicsContext graphicsContext) {
        int x, y;        //TODO Fix Explosion
        List<IEntity> gameplayObjects = runtimeInfo.getGameplayObjects();
        List<IEntity> explosionList = gameplayObjects.stream().filter(obj -> obj.getOBJECT_TYPE() == ObjectType.EXPLOSION)
                .toList();
        for (IEntity entity : explosionList) {
            y = (entity.getMapPosY()) / scaledTileSize;
            x = (entity.getMapPosX()) / scaledTileSize;
            if (TileManager.tileMap[y][x] > 19) {
                TileManager.tileMap[y][x] = 0;

                //spawning item entitys
                spawnItems(x * scaledTileSize, y * scaledTileSize, graphicsContext);
            }
        }
    }

    public void spawnExplosion(int spawnCordX, int spawnCordY, GraphicsContext graphicsContext) {
        List<IEntity> gameplayObjects = runtimeInfo.getGameplayObjects();
        IEntity explosion = EntityFactory.createGameplayObject(ObjectType.EXPLOSION, spawnCordY * scaledTileSize, spawnCordX * scaledTileSize, graphicsContext);
        gameplayObjects.add(explosion);
        runtimeInfo.setGameplayObjects(gameplayObjects);
        log.debug("explosion placed");
        changeCheckedTileDestructable(graphicsContext);
    }

    public void spawnItems(int x, int y, GraphicsContext graphicsContext) {
        final double dropChanceBigBombItemFrame = 0.1;
        final double dropChanceHeart = 0.15;
        final double dropChanceBlastUpItemFrame = 0.15;
        double randomDropChance = Math.random() * 1;

        if (randomDropChance < dropChanceBigBombItemFrame) {
            List<IEntity> gameplayObjects = runtimeInfo.getGameplayObjects();
            IEntity heart = EntityFactory.createGameplayObject(ObjectType.BIG_BOMB_ITEMFRAME, x, y, graphicsContext);
            gameplayObjects.add(heart);
            runtimeInfo.setGameplayObjects(gameplayObjects);
            log.info("heart dropped");
        } else if (randomDropChance < dropChanceHeart + dropChanceBigBombItemFrame) {
            List<IEntity> gameplayObjects = runtimeInfo.getGameplayObjects();
            IEntity bigBombItemFrame = EntityFactory.createGameplayObject(ObjectType.HEART, x, y, graphicsContext);
            gameplayObjects.add(bigBombItemFrame);
            runtimeInfo.setGameplayObjects(gameplayObjects);
            log.info("BigBombItemFrame dropped");
        } else if (randomDropChance < dropChanceHeart + dropChanceBigBombItemFrame + dropChanceBlastUpItemFrame) {
            List<IEntity> gameplayObjects = runtimeInfo.getGameplayObjects();
            IEntity bigBombItemFrame = EntityFactory.createGameplayObject(ObjectType.BLASTUP_ITEMFRAME, x, y, graphicsContext);
            gameplayObjects.add(bigBombItemFrame);
            runtimeInfo.setGameplayObjects(gameplayObjects);
            log.info("BlastUpItemFrame dropped");
        }
    }

    @Override
    public void attack(IEntity entity, GraphicsContext graphicsContext) {
        BoundingBox hitBox;

        //Added and subtracted numbers from variables are the pixel insets of the player sprite

        int attackRange = 30;
        int attackWidth = 10;

        if (inputHandler.isAttack() && PLAYER_MODE == PlayerMode.PLAYER_ONE && attackRate == 0 ||
                inputHandler.isSdAttack() && PLAYER_MODE == PlayerMode.PLAYER_TWO && attackRate == 0) {
            attackRate = 15;
            soundEffects.playSoundEffectOnce(SFX.SWORD_HIT);

            if (playerDirection == EntityDirection.UP) {
                hitBox = new BoundingBox(mapPosX + playerWidth, mapPosY - 10, attackWidth, attackRange);
                graphicsContext.strokeRect(mapPosX + playerWidth, mapPosY - 10, attackWidth, attackRange);
                graphicsContext.drawImage(swordUp, mapPosX + 8, mapPosY - 10, 32, 32);
                hitEnemy(entity, hitBox);
            } else if (playerDirection == EntityDirection.DOWN) {
                hitBox = new BoundingBox(mapPosX + playerWidth, mapPosY + playerHeight, attackWidth, attackRange);
                graphicsContext.strokeRect(mapPosX + playerWidth, mapPosY + playerHeight, attackWidth, attackRange);
                graphicsContext.drawImage(swordDown, mapPosX + 8, mapPosY + playerHeight, 32, 32);
                hitEnemy(entity, hitBox);
            } else if (playerDirection == EntityDirection.LEFT) {
                hitBox = new BoundingBox(mapPosX - attackWidth, mapPosY + ((double) playerHeight / 2),
                        attackRange, attackWidth);
                graphicsContext.strokeRect(mapPosX - attackWidth, mapPosY + ((double) playerHeight / 2),
                        attackRange, attackWidth);
                graphicsContext.drawImage(swordLeft, mapPosX - 8, mapPosY + 8, 32, 32);
                hitEnemy(entity, hitBox);
            } else {
                hitBox = new BoundingBox(mapPosX + playerWidth + attackWidth, mapPosY + ((double) playerHeight / 2),
                        attackRange, attackWidth);
                graphicsContext.strokeRect(mapPosX + playerWidth + attackWidth, mapPosY + ((double) playerHeight / 2),
                        attackRange, attackWidth);
                graphicsContext.drawImage(swordRight, mapPosX + playerWidth + 8, mapPosY + 8, 32, 32);
                hitEnemy(entity, hitBox);
            }
        }
    }

    private void hitEnemy(IEntity entity, BoundingBox hitBox) {
        if (hitBox.intersects(entity.getBoxCollider())) {
            entity.gotHit(damage);
            log.info("Hit enemy");
        }
    }

    @Override
    public void updateEntityWalkAnimation() {
        spriteCounter++;

        if (spriteCounter > 10) {
            if (spriteNumber == 1) {
                spriteNumber = 2;
            } else if (spriteNumber == 2) {
                spriteNumber = 1;
            }
            spriteCounter = 0;
        }
    }

    @Override
    public void renderEntity(GraphicsContext graphicsContext) {
        Image playerSprite = null;

        //for debugging collision
        graphicsContext.strokeRect(mapPosX + 15, mapPosY + 10, playerWidth, playerHeight);

        switch (playerDirection) {
            case UP:
                if (spriteNumber == 1) {
                    playerSprite = directionUpOne;
                }
                if (spriteNumber == 2) {
                    playerSprite = directionUpTwo;
                }
                break;
            case DOWN:
                if (spriteNumber == 1) {
                    playerSprite = directionDownOne;
                }
                if (spriteNumber == 2) {
                    playerSprite = directionDownTwo;
                }
                break;
            case LEFT:
                if (spriteNumber == 1) {
                    playerSprite = directionLeftOne;
                }
                if (spriteNumber == 2) {
                    playerSprite = directionLeftTwo;
                }
                break;
            case RIGHT:
                if (spriteNumber == 1) {
                    playerSprite = directionRightOne;
                }
                if (spriteNumber == 2) {
                    playerSprite = directionRightTwo;
                }
                break;
        }

        graphicsContext.drawImage(playerSprite, mapPosX, mapPosY, 48, 48);
    }

    @Override
    public void gotHit(int damageDone) {
        //soundEffects.playSoundEffectOnce();

        int damagedHealth = health - damageDone;
        if (iFrameCounter == 0) {
            if (damagedHealth > 0) {
                health -= damageDone;
                iFrameCounter = 10;
            } else {
                health = 0;
                //log.info("Dead");
            }
            //log.info("DamageTaken, current Health: " + health);
        }

    }

    @Override
    public void healPlayer(int healthRegenerated) {
        int regeneratedHealth = health + healthRegenerated;

        soundEffects.playSoundEffectOnce(SFX.POWERUP);

        if (regeneratedHealth < maxPlayerHealth && regeneratedHealth > 0) {

            health += healthRegenerated;
        } else {
            health = maxPlayerHealth;
        }
        log.info("Health: " + health);
    }

    @Override
    public BoundingBox getBoxCollider() {
        return boxCollider;
    }

    @Override
    public EntityDirection getEntityDirection() {
        return playerDirection;
    }

    @Override
    public int getEntitySpeed() {
        return playerSpeed;
    }

    @Override
    public int getMapPosX() {
        return mapPosX;
    }

    @Override
    public int getMapPosY() {
        return mapPosY;
    }

    @Override
    public ObjectType getOBJECT_TYPE() {
        return null;
    }

    @Override
    public ObjectStatus getOBJECT_STATUS() {
        return null;
    }

    @Override
    public void setObjectStatus(ObjectStatus objectStatus) {
    }

    @Override
    public int getHealth() {
        return health;
    }

    @Override
    public int getMaxEntityHealth() {
        return maxPlayerHealth;
    }

}